@@ -1,5 +1,4 @@
 from collections.abc import Iterable, Callable
-from functools import partial
 from typing import List
 from data_product_sdk.cortex_databricks.data_platform.aws_utils import S3
 from data_product_sdk.cortex_databricks.data_platform.databricks_utils import (
@@ -51,12 +50,13 @@ def __verify_parameters(function):
     def wrapper(*args, **kwargs):
         insert_modes_avalables = ["append", "upsert", "overwrite"]
         insert_mode = kwargs.get("insert_mode", "append")
-        if  insert_mode in insert_modes_avalables:
-             return function(*args, **kwargs)
+        if insert_mode in insert_modes_avalables:
+            return function(*args, **kwargs)
         else:
             raise ValueError(
                 f"Argument insert_mode is '{kwargs.get('insert_mode')}' but should be one of {insert_modes_avalables}"
             )
+
     return wrapper
 
 
@@ -147,16 +147,23 @@ def build_checkpoint_path(
     return checkpoint_location
 
 
-def read_table(spark_session: SparkSession, db_name: str, table_name: str, where=None) -> DataFrame:
+def read_table(
+    spark_session: SparkSession, db_name: str, table_name: str, where=None
+) -> DataFrame:
     table_name = f"{db_name}.{table_name}"
 
     if where is None:
-        df = spark_session.readStream.format('delta').table(tableName=table_name)
+        df = spark_session.readStream.format("delta").table(tableName=table_name)
     else:
-        df = spark_session.readStream.format('delta').table(tableName=table_name).filter(where)
+        df = (
+            spark_session.readStream.format("delta")
+            .table(tableName=table_name)
+            .filter(where)
+        )
 
     return df
 
+
 def read_data(
     spark_session: SparkSession,
     data_input_path: str,
@@ -213,10 +220,8 @@ def __write_function(
     micro_batch_df: DataFrame,
     db_name,
     table_name,
-    insert_mode: str,
     sort_fields: list,
     upsert_fields: list = None,
-    partition_columns: list = None,
     spark=None,
 ):
     if spark is None:
@@ -237,29 +242,14 @@ def __write_function(
     )
     table_name = f"{db_name}.{table_name}"
 
-    if insert_mode == "upsert":
-        try:
-            current_df = DeltaTable.forName(spark, table_name)
-        except Exception as e:
-            current_df = None
-    else:
-       current_df = None
+    current_df = DeltaTable.forName(spark, table_name)
 
-    if current_df:
-        current_df.alias(current_df_name).merge(
-            micro_batch_df.alias(micro_batch_df_name), upsert_condition
-        ).whenMatchedUpdateAll(match_condition).whenNotMatchedInsertAll().execute()
-    else:
-        if insert_mode == "overwrite":
-            truncate_table(spark=spark, db_table=table_name)
-        micro_batch_df.write.option("header", "true").option(
-            "mergeSchema", "true"
-        ).partitionBy(*partition_columns).format("delta").mode(insert_mode).saveAsTable(
-            table_name
-        )
-        spark.sql(f"ALTER TABLE {table_name} OWNER TO `data-platform`")
-        print(f"Created table {table_name} and loaded data")
-    print(f"Applied insert mode: {insert_mode}")
+    current_df.alias(current_df_name).merge(
+        micro_batch_df.alias(micro_batch_df_name), upsert_condition
+    ).whenMatchedUpdateAll(match_condition).whenNotMatchedInsertAll().execute()
+
+    spark.sql(f"ALTER TABLE {table_name} OWNER TO `data-platform`")
+    print(f"Created table {table_name} and loaded data")
 
     return True
 
@@ -290,26 +280,36 @@ def write_table(
         df.writeStream.trigger(once=True)
         .format("delta")
         .option("checkpointLocation", checkpoint_location)
+        .option("header", "true")
+        .option("mergeSchema", "true")
         .options(**write_options)
     )
 
-    # TODO: MODULARIZAR
+    table_exists = check_table_exists(
+        spark=spark, db_name=db_name, table_name=table_name
+    )
+
+    if insert_mode == "overwrite" and table_exists:
+        truncate_table(spark=spark, table_name=f"{db_name}.{table_name}")
+
     if sort_fields and upsert_fields:
-        if check_table_exists(spark=spark, db_name=db_name, table_name=table_name):
+        if table_exists:
             print(
-                f"TABELA EXISTE - upsert_fields: {upsert_fields} - sort_fields: {sort_fields}"
+                f"TABLE EXIST - upsert_fields: {upsert_fields} - sort_fields: {sort_fields}"
             )
-            upsert_func = partial(
-                __write_function,
-                db_name=db_name,
-                table_name=table_name,
-                insert_mode=insert_mode,
-                sort_fields=sort_fields,
-                upsert_fields=upsert_fields,
-                partition_columns=partition_columns,
-                spark=spark,
-            )
-            return stream.foreachBatch(upsert_func).start()
+
+            return stream.foreachBatch(
+                lambda df, epochId: __write_function(
+                    micro_batch_df=df,
+                    db_name=db_name,
+                    table_name=table_name,
+                    sort_fields=sort_fields,
+                    upsert_fields=upsert_fields,
+                    spark=spark,
+                )
+                if not df.rdd.isEmpty()
+                else None
+            ).start()
         else:
             stream = stream.toTable(
                 tableName=f"{db_name}.{table_name}", partitionBy=partition_columns
@@ -396,9 +396,10 @@ def set_owner_to(spark, db_table, owner: str):
 
     return True
 
-def truncate_table(spark, db_table):
-    spark.sql(f"TRUNCATE TABLE {db_table}")
-    print(f"TABLE {db_table} truncated")
+
+def truncate_table(spark, table_name):
+    spark.sql(f"TRUNCATE TABLE {table_name}")
+    print(f"TABLE {table_name} truncated")
 
     return True
 
@@ -518,7 +519,7 @@ def landing_to_bronze(
         spark=spark,
         partition_columns=partition_columns,
         write_options=write_options,
-        upsert_fields=upsert_fields
+        upsert_fields=upsert_fields,
     )
 
     db_table = f"{db_name}.{dataset_name}"
@@ -536,6 +537,7 @@ def landing_to_bronze(
 
     return True
 
+
 @__verify_parameters
 def bronze_to_silver(
     input_table_name,
@@ -556,8 +558,7 @@ def bronze_to_silver(
     where=None,
     tbl_properties=None,
     sort_fields: list = None,
-    upsert_fields: list = None
-
+    upsert_fields: list = None,
 ):
     if spark is None:
         spark = get_spark_session(app_name=dataset_name)
@@ -593,7 +594,9 @@ def bronze_to_silver(
             spark=spark, db_name=input_db_name, table_name=input_table_name
         )
 
-    df = read_table(spark, db_name=input_db_name, table_name=input_table_name, where=where)
+    df = read_table(
+        spark, db_name=input_db_name, table_name=input_table_name, where=where
+    )
     df = apply_processing_function(
         df=df,
         func=processing_function,
@@ -617,7 +620,7 @@ def bronze_to_silver(
         partition_columns=partition_columns,
         write_options=write_options,
         sort_fields=sort_fields,
-        upsert_fields=upsert_fields
+        upsert_fields=upsert_fields,
     )
 
     output_db_table = f"{output_db_name}.{dataset_name}"
@@ -656,7 +659,7 @@ def silver_to_gold(
     reset_checkpoint=False,
     catalog_name=None,
     where=None,
-    tbl_properties=None
+    tbl_properties=None,
 ):
     if spark is None:
         spark = get_spark_session(app_name=dataset_name)
@@ -667,7 +670,7 @@ def silver_to_gold(
     if input_db_name is None:
         input_db_name = SILVER_DB_NAME
     if output_db_name is None:
-        output_db_name=GOLD_DB_NAME
+        output_db_name = GOLD_DB_NAME
 
     if catalog_name is None:
         catalog_name = "hive_metastore"
@@ -692,7 +695,9 @@ def silver_to_gold(
     if insert_mode is None:
         insert_mode = "append"
 
-    df = read_table(spark, db_name=input_db_name, table_name=input_table_name, where=where)
+    df = read_table(
+        spark, db_name=input_db_name, table_name=input_table_name, where=where
+    )
     df = apply_processing_function(
         df=df,
         func=processing_function,
@@ -746,7 +751,7 @@ def gold_to_kafka(
     processing_function_kwargs=None,
     reset_checkpoint=False,
     catalog_name=None,
-    where=None
+    where=None,
 ):
     topics = [topic_name]
     create_topics_if_not_exists(
@@ -776,7 +781,9 @@ def gold_to_kafka(
             spark=spark, db_name=input_db_name, table_name=input_table_name
         )
 
-    df = read_table(spark, db_name=input_db_name, table_name=input_table_name, where=where)
+    df = read_table(
+        spark, db_name=input_db_name, table_name=input_table_name, where=where
+    )
     df = apply_processing_function(
         df=df,
         func=processing_function,
